package xbtplotftpbin;

import java.util.Vector;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Comparator;
import java.util.Date;

public class ProfileResult implements Comparable<ProfileResult> {

    private String callSign;
    private String fileName;
    private int instrumentType;
    private double latitude;
    private double longitude;
    private double maxDepth;
    private double maxTemperature;
    private double minTemperature;
    private double nextPOS;
    private Date observationDateTime;
    private int observationDay;
    private int observationHour;
    private int observationMinute;
    private int observationMonth;
    private int observationYear;
    private double previousPOS;
    private Date recoveryDateTime;
    private int recoveryDay;
    private int recoveryHour;
    private int recoveryMinute;
    private int recoveryMonth;
    private int recoveryYear;
    private String soopLine;
    private Vector temperaturePoints;
    private int thisDataIs;
    private int xLocation;
    private int yLocation;

    /* valid profile resolutions */
    private static final int FULLRESOLUTION = 1;
    private static final int TWOMETERS = 2;
    private static final int INFLECTIONPOINTS = 3;

    /* data & meta-data precision values */
    private static final int TWODEC = 2;

    public void ProfileResult() {
        setMaxDepth(0);
        setMaxTemperature(-10);
        setMinTemperature(99);
        setPreviousPOS(-1);
        setNextPOS(-1);
    }

    @Override
    public int compareTo(ProfileResult t) {
        return getObservationDateTime().compareTo(t.getObservationDateTime());
    }

    public String getCallSign() {return callSign;}
    public String getFileName() {return fileName;}
    public int getInstrumentType() {return instrumentType;}
    public double getLatitude() {return latitude;}
    public double getLongitude() {return longitude;}
    public double getMaxDepth() {return maxDepth;}
    public double getMaxTemperature() {return maxTemperature;}
    public double getMinTemperature() {return minTemperature;}
    public double getNextPOS() {return nextPOS;}
    public Date getObservationDateTime() {return observationDateTime;}
    public int getObservationDay() {return observationDay;}
    public int getObservationHour() {return observationHour;}
    public int getObservationMinute() {return observationMinute;}
    public int getObservationMonth() {return observationMonth;}
    public int getObservationYear() {return observationYear;}
    public double getPreviousPOS() {return previousPOS;}
    public Date getRecoveryDateTime() {return recoveryDateTime;}
    public int getRecoveryDay() {return recoveryDay;}
    public int getRecoveryHour() {return recoveryHour;}
    public int getRecoveryMinute() {return recoveryMinute;}
    public int getRecoveryMonth() {return recoveryMonth;}
    public int getRecoveryYear() {return recoveryYear;}
    public String getSoopLine() {return soopLine;}
    public Vector getTemperaturePoints() {return temperaturePoints;}
    public int getThisDataIs() {return thisDataIs;}
    public int getxLocation() {return xLocation;}
    public int getyLocation() {return yLocation;}

    public void setCallSign(String cs) {callSign = cs;}
    public void setFileName(String fn) {fileName = fn;}
    public void setFileNameAndRecoveryDate(String fn) {
        fileName = fn;
        String[] tokens = fn.split("_");
        int underScoreCount = tokens.length;
        String receivedTime = tokens[1];
        if(underScoreCount < 3) {
            setRecoveryYear(0000);
            setRecoveryMonth(0);
            setRecoveryDay(0);
            setRecoveryHour(0);
            setRecoveryMinute(0);
            setRecoveryDateTime();
        } else {
            if(receivedTime.indexOf(".")>=0) {
                receivedTime = tokens[1].replace(".", "") +
                        "-" + tokens[2].replace(".", "");
            }
            constructDateFromFileName(receivedTime);
        }
    }
    public void setInstrumentType(int it) {instrumentType = it;}
    public void setLatitude(double l) {latitude = l;}
    public void setLongitude(double l) {longitude = l;}
    public void setMaxDepth(double md) {maxDepth = md;}
    public void setMaxTemperature(double mt) {maxTemperature = mt;}
    public void setMinTemperature(double mt) {minTemperature = mt;}
    public void setNextPOS(double pos) {nextPOS = pos;}
    public void setObservationDateTime() {
       observationDateTime =
               profileDateTimeFormat(getObservationYear(),
               getObservationMonth(), getObservationDay(),
               getObservationHour(), getObservationMinute());
    }
    public void setObservationDateTime(Date odt) {observationDateTime = odt;}
    public void setObservationDay(int d) {observationDay = d;}
    public void setObservationHour(int hr) {observationHour = hr;}
    public void setObservationMonth(int m) {observationMonth = m;}
    public void setObservationMinute(int m) {observationMinute = m;}
    public void setObservationYear(int yr) {observationYear = yr;}
    public void setPreviousPOS(double pos) {previousPOS = pos;}
    public void setRecoveryDateTime(Date rdt) {recoveryDateTime = rdt;}
    public void setRecoveryDateTime() {
        recoveryDateTime =
                profileDateTimeFormat(getRecoveryYear(),
                        getRecoveryMonth(), getRecoveryDay(),
                        getRecoveryHour(), getRecoveryMinute());
    }
    public void setRecoveryDay(int d) {recoveryDay = d;}
    public void setRecoveryHour(int hr) {recoveryHour = hr;}
    public void setRecoveryMonth(int m) {recoveryMonth = m;}
    public void setRecoveryMinute(int m) {recoveryMinute = m;}
    public void setRecoveryYear(int yr) {recoveryYear = yr;}
    public void setSoopLine(String sl) {soopLine = sl;}
    public void setTemperaturePoints(Vector tp) {temperaturePoints = tp;}
    public void setThisDataIs(int tdi) {thisDataIs = tdi;}
    public void setxLocation(int xl) {xLocation = xl;}
    public void setyLocation(int yl) {yLocation = yl;}

    public void distanceToNextPOS(ProfileResult nextPR) {
        DistanceCalculator dc = new DistanceCalculator(getLatitude(),
                getLongitude(), nextPR.getLatitude(), nextPR.getLongitude());
        setNextPOS(dc.getGreatCircleDistance());
    }

    public void distanceToPreviousPOS(ProfileResult prevPR) {
        DistanceCalculator dc = new DistanceCalculator(getLatitude(),
                getLongitude(), prevPR.getLatitude(), prevPR.getLongitude());
        setPreviousPOS(dc.getGreatCircleDistance());
    }

    public void constructDateFromFileName(String rt) {
        setRecoveryYear(
                Integer.parseInt(rt.substring(0, 4)));
        setRecoveryMonth(
                Integer.parseInt(rt.substring(4, 6)));
        setRecoveryDay(
                Integer.parseInt(rt.substring(6, 8)));
        setRecoveryHour(
                Integer.parseInt(rt.substring(9, 11)));
        setRecoveryMinute(
                Integer.parseInt(rt.substring(11)));
        setRecoveryDateTime();
    }

    public Date profileDateTimeFormat(int yr, int mon, int d, int hr, int min) {
        SimpleDateFormat fileDateFormat =
                new SimpleDateFormat("yyyyMMddHHmm");
        String stringDate =
                String.format("%04d", yr) +
                String.format("%02d", mon) +
                String.format("%02d", d) +
                String.format("%02d", hr) +
                String.format("%02d", min);
        try {
            return fileDateFormat.parse(stringDate);
        } catch (ParseException ex) {
            return null;
        }
    }

    public String profileDateTimeFormatted(Date pdt, String timezone) {
        SimpleDateFormat odtf = 
            new SimpleDateFormat ("yyyy-MM-dd HH:mm ");
        return odtf.format(pdt) + timezone;
    }

    public boolean temperatureObservations(double[] temps) {
        int counter = 0;
        for (double tm : temps) {
            double d = 0;
            if(getThisDataIs() == FULLRESOLUTION) {
                d = XBTDepthCalculator
                        .calculateDepth(getInstrumentType(), (counter+1)*0.1);
            } else if(getThisDataIs() == TWOMETERS) {
                d = 2 * counter;
            } else if(getThisDataIs() == INFLECTIONPOINTS) {
                System.out.println("Profile using inflection points: " +
                        getFileName());
                return false;
            } else {
                System.out.println("Profile resolution type " + getThisDataIs()
                        + " not found: " + getFileName());
                return false;
            }
            
            if (d > getMaxDepth()) {
                setMaxDepth(d);
            }
            if (tm > getMaxTemperature()) {
                setMaxTemperature(tm);
            }
            if (tm < getMinTemperature()) {
                setMinTemperature(tm);
            }

            DTPair pt = new DTPair(d, tm);
            getTemperaturePoints().addElement(pt);
            counter = counter + 1;
        }
        return true;
    }

    public static class Comparators {
        public static Comparator<ProfileResult> MAXDEPTH =
                new Comparator<ProfileResult>() {
            @Override
            public int compare(ProfileResult pr1, ProfileResult pr2) {
                if(pr1.getMaxDepth() < pr2.getMaxDepth()) return -1;
                if(pr1.getMaxDepth() > pr2.getMaxDepth()) return 1;
                return 0;
            }
        };

        public static Comparator<ProfileResult> MAXTEMPERATURE =
                new Comparator<ProfileResult>() {
            @Override
            public int compare(ProfileResult pr1, ProfileResult pr2) {
                if(pr1.getMaxTemperature() < pr2.getMaxTemperature()) return -1;
                if(pr1.getMaxTemperature() > pr2.getMaxTemperature()) return 1;
                return 0;
            }
        };

        public static Comparator<ProfileResult> MINTEMPERATURE =
                new Comparator<ProfileResult>() {
            @Override
            public int compare(ProfileResult pr1, ProfileResult pr2) {
                if(pr1.getMinTemperature() < pr2.getMinTemperature()) return -1;
                if(pr1.getMinTemperature() > pr2.getMinTemperature()) return 1;
                return 0;
            }
        };
    }
}


